/*  Sketch zur Darstellung von Elementarzelle und Gitter eines Diamanten
Zu Beginn wird eine Elementarzelle gezeichnet
Falls das Programmfenster aktiviert ist (darauf klicken) können durch entsprechenden Tastendruck
folgende Befehle ausgeführt werden:
Taste '1'        Eine Elementarzelle (EZ) 
Taste '2'        2^3 = 8 EZ
Taste '3'        3^3 = 27 EZ
Taste 'o'        obere 4 Tetraeder zeichnen / löschen
Taste 'u'        untere 4 Tetraeder zeichnen / löschen
Taste 'i'        innere Oktaeder rot zeichnen / löschen
Taste 'p'        obere dreistigen Pyramiden zeichnen / löschen
Taste 'q'        untere dreistigen Pyramiden zeichnen / löschen
Taste 'm'        mittlere dreistigen Pyramiden zeichnen / löschen
Taste 't'        Tetraeder bzw Pyramiden transparent / nicht transparent
Taste 'k'        Würfelkanten einer EZ löschen / zeichnen
Mit dem Mausrad hinein- bzw herauszoomen
Man kann leicht zeigen, das die Volumina von Tetraeder und Oktaeder in einem Diamantgitter
sich wie 2:7 verhalten.
Erzänzung zu "Programmieren lernen mit Computergrafik" - Oliver Deußen,Thomas Ningelgen
*/
import peasy.*;
PeasyCam cam;
float s;
float t;
float r;
float rDiag;
float w;
float abstand;
color[]f = new color[8];//Farbe für die Tetraeder
color[]g = new color[12];//Farbe für die Py
boolean kZ;
boolean tetraederUnten,tetraederOben;
boolean tetraederMitAtom;
boolean oktaederZeichnen;
boolean pyramidenZeichnenOben,pyramidenZeichnenUnten,pyramidenZeichnenMitte;
boolean eineEZ, achtEZ, siebenundzwanzigEZ;
boolean transparent;


void setup() {
  size(1200, 1200, P3D);
  s = 150;//Seitenlänge der Elementarzelle
  t=s/2;//um Schreibarbeit zu sparen
  r =4;//Kantenlänge der Würfel
  w =3; //Breite der Verbindungen
  abstand =0;
  kZ = true;//Kanten des Würfels werden gezeichnet
  tetraederUnten = false;//acht Tetraeder unten anzeigen-nicht anzeigen
  tetraederOben=false;//acht Tetraeder oben anzeigen-nicht anzeigen
  tetraederMitAtom=true;
  oktaederZeichnen = false;//Oktaeder innen
  transparent=false;
  pyramidenZeichnenOben = false;
  pyramidenZeichnenUnten = false;
  pyramidenZeichnenMitte = false;
  eineEZ = true;//eine Elementarzelle
  achtEZ = false;
  siebenundzwanzigEZ = false;//Gitter mit 27 EZ
  cam = new PeasyCam(this, 600);
  //Farben Tetraeder mit Gewicht auf grün. Blau und rot für Oktaeder
  for(int i = 0; i<8;i++) f[i] = color(random(0,200),random(200,255),random(0,200)); 
  for(int i = 0; i<12;i++)g[i] = color(random(0,200),random(0,200),random(200,255));  
}

void draw(){
  background(255);
  lights();
  if(eineEZ)gitterZeichnen(0,0,0);
    else if(achtEZ){
      for(int i=0; i<2;i++){
        for(int j=0; j<2;j++){
          for(int k=0; k<2;k++){
              gitterZeichnen(i,j,k);
      }      
        }    
           } 
    }
    else if(siebenundzwanzigEZ=true){
      for(int i=-1; i<2;i++){
        for(int j=-1; j<2;j++){
          for(int k=-1; k<2;k++){
           gitterZeichnen(i,j,k);
      }      
         }    
           } 
   }
}

void gitterZeichnen(int X, int Y, int Z){
  translate(X*s,Y*s,Z*s);
    noFill();
  if(kZ) stroke(0);
  else noStroke();
  box(s);
  //strokeWeight(w);
  verbindungenZeichnen();  
  strokeWeight(1);
  stroke(0);
  atomeZeichnen(); 
  alleTetraederZeichnen();
  tetraederMitAtomZeichnen();
  oktaederZeichnen();
  pyramidenZeichnen();
  translate(-X*s,-Y*s,-Z*s);
}

void verbindungenZeichnen(){
  strokeWeight(w);
  stroke(60);
  line(-s/2,s/2,-s/2,-s/4,s/4,-s/4);
  line(0,s/2,0,-s/4,s/4,-s/4);
  line(-s/2,0,0,-s/4,s/4,-s/4);
  line(0,0,-s/2,-s/4,s/4,-s/4);
  
  line(s/2,-s/2,-s/2,s/4,-s/4,-s/4);
  line(0,-s/2,0,s/4,-s/4,-s/4);
  line(0,0,-s/2,s/4,-s/4,-s/4);
  line(s/2,0,0,s/4,-s/4,-s/4);
  
  line(s/2,s/2,s/2,s/4,s/4,s/4);
  line(0,s/2,0,s/4,s/4,s/4);
  line(s/2,0,0,s/4,s/4,s/4);
  line(0,0,s/2,s/4,s/4,s/4);
  
  line(-s/2,-s/2,s/2,-s/4,-s/4,s/4);
  line(0,-s/2,0,-s/4,-s/4,s/4);
  line(-s/2,0,0,-s/4,-s/4,s/4);
  line(0,0,s/2,-s/4,-s/4,s/4);
}

void atomeZeichnen(){
  //Vier Atome auf den Ecken des Würfels
  pushMatrix();
    fill(0,255,0);
    stroke(0,255,0);
    translate(s/2,s/2,s/2);
    strokeWeight(2);
    box(r);
    translate(-s,0,-s);
    box(r);
    translate(0,-s,s);
    box(r);
    translate(s,0,-s);
    box(r);
  popMatrix();
  //Sechs Atome zentriert auf den Flächen des Würfels
   pushMatrix();
    stroke(0,0,255);
    fill(0,0,255);
    translate(0,0,s/2);
    box(r);
    translate(0,0,-s);
    box(r);
    translate(-s/2,0,s/2);
    box(r);
    translate(s,0,0);
    box(r);
    translate(-s/2,s/2,0);
    box(r);
    translate(0,-s,0);
    box(r);
  popMatrix();
  //Vier Atome auf 1/4 bzw 3/4 Raumdiagonale
   pushMatrix();
    stroke(255,0,0);
    translate(-0.25*s,-0.25*s,0.25*s);
    box(r);
    translate(0.5*s,0,-0.5*s);
    box(r);
    translate(0,0.5*s,0.5*s);
    box(r);
    translate(-0.5*s,0,-0.5*s);
    box(r);
  popMatrix();
}

void alleTetraederZeichnen(){
  stroke(0);
  beginShape(TRIANGLES); 
  
  if(tetraederOben){
    //Tetraeder links oben
    if(!transparent)fill(f[0]);else fill(f[0],50);
    vertex(-t,-t,t);vertex(0,0,t);vertex(0,-t,0);
    vertex(-t,-t,t);vertex(-t,0,0);vertex(0,0,t);
    vertex(-t,-t,t);vertex(-t,0,0);vertex(0,-t,0);
    vertex(-t,0,0);vertex(0,0,t);vertex(0,-t,0);
    //Tetraeder rechts oben
    if(!transparent)fill(f[1]);else fill(f[1],50);
    vertex(t,-t,-t);vertex(0,-t,0);vertex(t,0,0);
    vertex(t,-t,-t);vertex(0,-t,0);vertex(0,0,-t);
    vertex(t,-t,-t);vertex(t,0,0);vertex(0,0,-t);
    vertex(0,-t,0);vertex(t,0,0);vertex(0,0,-t);     
   //Tetraeder rechts oben
    if(!transparent)fill(f[2]);else fill(f[2],50);
    vertex(t,-t,t);vertex(0,0,t);vertex(t,0,0);
    vertex(t,-t,t);vertex(0,0,t);vertex(0,-t,0);
    vertex(t,-t,t);vertex(t,0,0);vertex(0,-t,0);
    vertex(0,0,t);vertex(t,0,0);vertex(0,-t,0);     
   //Tetraeder links oben
    if(!transparent)fill(f[3]);else fill(f[3],50);
    vertex(-t,-t,-t);vertex(-t,0,0);vertex(0,-t,0);
    vertex(-t,-t,-t);vertex(-t,0,0);vertex(0,0,-t);
    vertex(-t,-t,-t);vertex(0,-t,0);vertex(0,0,-t);
    vertex(-t,0,0);vertex(0,-t,0);vertex(0,0,-t);
  }
  if(tetraederUnten){ //Tetraeder unten
  //Tetraeder links hinten
    if(!transparent)fill(f[4]);else fill(f[4],50);
    vertex(-t,t,-t);vertex(-t,0,0);vertex(0,t,0);
    vertex(-t,t,-t);vertex(-t,0,0);vertex(0,0,-t);
    vertex(-t,t,-t);vertex(0,t,0);vertex(0,0,-t);
    vertex(-t,0,0);vertex(0,t,0);vertex(0,0,-t);
    //Tetraeder rechts vorn
    if(!transparent)fill(f[5]);else fill(f[5],50);
    vertex(t,t,t);vertex(0,t,0);vertex(0,0,t);
    vertex(t,t,t);vertex(0,t,0);vertex(t,0,0);
    vertex(t,t,t);vertex(0,0,t);vertex(t,0,0);
    vertex(0,t,0);vertex(0,0,t);vertex(t,0,0);
    //Tetraeder rechts hinten
    if(!transparent)fill(f[6]);else fill(f[6],50);
    vertex(t,t,-t);vertex(0,t,0);vertex(t,0,0);
    vertex(t,t,-t);vertex(0,t,0);vertex(0,0,-t);
    vertex(t,t,-t);vertex(0,0,-t);vertex(t,0,0);
    vertex(0,t,0);vertex(t,0,0);vertex(0,0,-t);
    //Tetraeder links vorn
    if(!transparent)fill(f[7]);else fill(f[7],50);
    vertex(-t,t,t);vertex(-t,0,0);vertex(0,0,t);
    vertex(-t,t,t);vertex(-t,0,0);vertex(0,t,0);
    vertex(-t,t,t);vertex(0,0,t);vertex(0,t,0);
    vertex(0,0,t);vertex(-t,0,0);vertex(0,t,0);    
  }   
  endShape();  
}

void tetraederMitAtomZeichnen(){
  if(tetraederMitAtom){
  beginShape(TRIANGLES); 
    //Tetraeder links oben vorn
    if(!transparent)fill(f[0]);else fill(f[0],50);
    vertex(-t,-t,t);vertex(0,0,t);vertex(0,-t,0);
    vertex(-t,-t,t);vertex(-t,0,0);vertex(0,0,t);
    vertex(-t,-t,t);vertex(-t,0,0);vertex(0,-t,0);
    vertex(-t,0,0);vertex(0,0,t);vertex(0,-t,0);
    
    //Tetraeder rechts oben hinten
    if(!transparent)fill(f[1]);else fill(f[1],50);
    vertex(t,-t,-t);vertex(0,-t,0);vertex(t,0,0);
    vertex(t,-t,-t);vertex(0,-t,0);vertex(0,0,-t);
    vertex(t,-t,-t);vertex(t,0,0);vertex(0,0,-t);
    vertex(0,-t,0);vertex(t,0,0);vertex(0,0,-t);     
    
    if(!transparent)fill(f[2]);else fill(f[2],50);
    vertex(-t,t,-t);vertex(-t,0,0);vertex(0,t,0);
    vertex(-t,t,-t);vertex(-t,0,0);vertex(0,0,-t);
    vertex(-t,t,-t);vertex(0,t,0);vertex(0,0,-t);
    vertex(-t,0,0);vertex(0,t,0);vertex(0,0,-t);
    
     //Tetraeder rechts vorn
    if(!transparent)fill(f[5]);else fill(f[5],50);
    vertex(t,t,t);vertex(0,t,0);vertex(0,0,t);
    vertex(t,t,t);vertex(0,t,0);vertex(t,0,0);
    vertex(t,t,t);vertex(0,0,t);vertex(t,0,0);
    vertex(0,t,0);vertex(0,0,t);vertex(t,0,0);
  endShape(); 
  }
}

void oktaederZeichnen(){
 if(oktaederZeichnen){
    stroke(0);
    beginShape(TRIANGLES);  
    fill(255,0,0);
      vertex(-t,0,0);vertex(0,0,t);vertex(0,-t,0);
      vertex(0,-t,0);vertex(t,0,0);vertex(0,0,-t);
      vertex(0,0,t);vertex(t,0,0);vertex(0,-t,0); 
      vertex(-t,0,0);vertex(0,-t,0);vertex(0,0,-t);
      vertex(-t,0,0);vertex(0,t,0);vertex(0,0,-t);
      vertex(0,t,0);vertex(0,0,t);vertex(t,0,0);
      vertex(0,t,0);vertex(t,0,0);vertex(0,0,-t);
      vertex(0,0,t);vertex(-t,0,0);vertex(0,t,0);  
    endShape();  
 }
}

void pyramidenZeichnen(){
  if(pyramidenZeichnenOben){
     beginShape(TRIANGLES); 
         if(!transparent)fill(g[0]);else fill(g[0],50);;//oben rechts
         vertex(0,-t,0);vertex(t,0,0);vertex(t,-t,-t);
         vertex(0,-t,0);vertex(t,0,0);vertex(t,-t,t);
         vertex(0,-t,0);vertex(t,-t,-t);vertex(t,-t,t);
         vertex(t,0,0);vertex(t,-t,-t);vertex(t,-t,t);
         if(!transparent)fill(g[1]);else fill(g[1],50);//oben vorn
         vertex(0,-t,0);vertex(0,0,t);vertex(t,-t,t);
         vertex(0,-t,0);vertex(0,0,t);vertex(-t,-t,t);
         vertex(0,-t,0);vertex(-t,-t,t);vertex(t,-t,t);
         vertex(t,-t,t);vertex(-t,-t,t);vertex(0,0,t);
         if(!transparent)fill(g[2]);else fill(g[2],50);//oben links
         vertex(0,-t,0);vertex(-t,-t,-t);vertex(-t,0,0);
         vertex(0,-t,0);vertex(-t,-t,t);vertex(-t,0,0);
         vertex(0,-t,0);vertex(-t,-t,-t);vertex(-t,-t,t);
         vertex(-t,0,0);vertex(-t,-t,-t);vertex(-t,-t,t);
         if(!transparent)fill(g[3]);else fill(g[3],50);
         vertex(0,-t,0);vertex(0,0,-t);vertex(-t,-t,-t);
         vertex(0,-t,0);vertex(0,0,-t);vertex(t,-t,-t);
         vertex(-t,-t,-t);vertex(t,-t,-t);vertex(0,-t,0);
         vertex(-t,-t,-t);vertex(t,-t,-t);vertex(0,0,-t);
     endShape();      
  }
  if(pyramidenZeichnenUnten){
     beginShape(TRIANGLES); 
         if(!transparent)fill(g[4]);else fill(g[4],50);//oben rechts
         vertex(0,t,0);vertex(t,0,0);vertex(t,t,-t);
         vertex(0,t,0);vertex(t,0,0);vertex(t,t,t);
         vertex(0,t,0);vertex(t,t,-t);vertex(t,t,t);
         vertex(t,0,0);vertex(t,t,-t);vertex(t,t,t);
         if(!transparent)fill(g[5]);else fill(g[5],50);//oben vorn
         vertex(0,t,0);vertex(0,0,t);vertex(t,t,t);
         vertex(0,t,0);vertex(0,0,t);vertex(-t,t,t);
         vertex(0,t,0);vertex(-t,t,t);vertex(t,t,t);
         vertex(0,0,t); vertex(t,t,t);vertex(-t,t,t);
         if(!transparent)fill(g[6]);else fill(g[6],50);//oben links
         vertex(-t,0,0);vertex(0,t,0);vertex(-t,t,-t);
         vertex(-t,0,0);vertex(0,t,0);vertex(-t,t,t);
         vertex(0,t,0);vertex(-t,t,-t);vertex(-t,t,t);
         vertex(-t,0,0);vertex(-t,t,-t);vertex(-t,t,t);
         if(!transparent)fill(g[7]);else fill(g[7],50);//oben hinten
         vertex(0,t,0);vertex(0,0,-t);vertex(-t,t,-t);
         vertex(0,t,0);vertex(0,0,-t);vertex(t,t,-t);
         vertex(-t,t,-t);vertex(t,t,-t);vertex(0,t,0);
         vertex(-t,t,-t);vertex(t,t,-t);vertex(0,0,-t);         
     endShape();      
  }
  if(pyramidenZeichnenMitte){
     beginShape(TRIANGLES); 
         if(!transparent)fill(g[8]);else fill(g[8],50);
         vertex(0,0,t);vertex(-t,0,0);vertex(-t,-t,t);
         vertex(0,0,t);vertex(-t,0,0);vertex(-t,t,t);
         vertex(0,0,t);vertex(-t,-t,t);vertex(-t,t,t);
         vertex(-t,0,0);vertex(-t,-t,t);vertex(-t,t,t);
         if(!transparent)fill(g[9]);else fill(g[9],50);
         vertex(t,0,0);vertex(0,0,t);vertex(t,-t,t);
         vertex(t,0,0);vertex(0,0,t);vertex(t,t,t);
         vertex(t,0,0);vertex(t,-t,t);;vertex(t,t,t);
         vertex(0,0,t);vertex(t,-t,t);;vertex(t,t,t);
         if(!transparent)fill(g[10]);else fill(g[10],50);
         vertex(-t,0,0);vertex(0,0,-t);vertex(-t,t,-t);
         vertex(-t,0,0);vertex(0,0,-t);vertex(-t,-t,-t);
         vertex(-t,0,0);vertex(-t,t,-t);vertex(-t,-t,-t);
         vertex(0,0,-t);vertex(-t,t,-t);vertex(-t,-t,-t);
         if(!transparent)fill(g[11]);else fill(g[11],50);
         vertex(t,0,0);vertex(0,0,-t);vertex(t,-t,-t);
         vertex(t,0,0);vertex(0,0,-t);vertex(t,t,-t);
         vertex(t,0,0);vertex(t,-t,-t);vertex(t,t,-t);
         vertex(0,0,-t);vertex(t,-t,-t);vertex(t,t,-t);
     endShape();      
  }
}

void keyPressed(){
   if (key == 'k')kZ=!kZ;
   if (key == 'u')tetraederUnten=!tetraederUnten;
   if (key == 'o')tetraederOben=!tetraederOben;
   if (key == 'a')tetraederMitAtom=!tetraederMitAtom;
   if (key == 'i')oktaederZeichnen=!oktaederZeichnen;
   if (key == 'p')pyramidenZeichnenOben=!pyramidenZeichnenOben;
   if (key == 'q')pyramidenZeichnenUnten=!pyramidenZeichnenUnten;
   if (key == 'm')pyramidenZeichnenMitte=!pyramidenZeichnenMitte;
   if (key == '1')eineEZ = true;
   if (key == '2'){achtEZ = true;eineEZ=false;}
   if (key == '3'){eineEZ = false;achtEZ = false; siebenundzwanzigEZ=true;}
   if (key == 't')transparent=!transparent;
   }
